function [T3] = multihtest(r,rboots,sboots,gboots,K2) 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MATLAB code: Section 11.6.1 (Quantile residuals)
% File: multihtest.m
% Coded by: JDG (converted from GAUSS code written by Leena Kalliovirta) 
%
% Heteroskedasticity test statistic based on MULTIVARIATE (dimension of the data
% is denoted by "m") quantile residuals (QRs)
%              
% INPUT:     
% r      = vector of quantile residuals (T * m)        
% rboots = simulated vector of QRs using estimated model (Tboots * m). 
%          with Tboots some large number (say Tboots = 20,000).
% sboots = simulated matrix of scores of QRs using estimated model 
%          (Tboots * k) with k the number of columns of sboots.
% gboots = simulated matrix of derivatives of QRs using estimated models 
%          (Tboots * k).                                    
% K2     = number of lags used in the heteroskedasticity test. 
%            
% OUTPUT: 
% T3     = scalar value of the test statistic, follows a Chi^{2}(m^2 K2) 
%          distribution under H0: no heteroskedasticity in multivariate QRs
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%               
T      = length(r);
Tboots = length(rboots);
[~,k]  = size(sboots); 
[~,m]  = size(r);

warning('off','all')  
warning
% Disable all warnings. Replace 'off' by 'on' to enable
% Warning: Matrix "cov" may be close to singular or badly scaled.
%          Results may be inaccurate. 

cov    = inv((sboots'*sboots)/Tboots);
gu     = zeros(T-K2,K2*m^2);

for t1=1:(T-K2)
  help     = ((r(t1+K2,:).^2-1)'*(r(t1+K2-1,:).^2-1));
  for i=2:K2
    help   = [help;((r(t1+K2,:).^2-1)'*(r(t1+K2-i,:).^2-1))];
  end;
  gu(t1,:) = reshape(help,[K2*m^2,1]); 
end;

RRR       = mean(gu);
guboots   = zeros(Tboots-K2,K2*m^2);

for t1=1:(Tboots-K2)
  helpboots     = ((rboots(t1+K2,:).^2-1)'*(rboots(t1+K2-1,:).^2-1));
  for i=2:K2
    helpboots   = [helpboots;((rboots(t1+K2,:).^2-1)'*(rboots(t1+K2-i,:).^2-1))];
  end;
  guboots(t1,:) = reshape(helpboots,[K2*m^2,1]); 
end;

G3  = zeros(K2*m^2,k);
for j=1:K2
  for b=1:m
    for a=1:m   
    rmat1 = (rboots((K2+1-j):(Tboots-j),b).^2-1);     
    rmat2 = rmat1.*rboots((K2+1):Tboots,a);
    repr2 = repmat(rmat2,1,k);           % repeat copies of array
    Crrg1 = repr2.*gboots((K2+1):Tboots,((a-1)*k+1):a*k);
    
    rmat3 = (rboots((K2+1):Tboots,a).^2-1);  
    rmat4 = rmat3.*rboots((K2+1-j):(Tboots-j),b);
    repr4 = repmat(rmat4,1,k);           % repeat copies of array   
    Crrg2 = repr4.*gboots((K2+1-j):(Tboots-j),((b-1)*k+1):b*k);    
    
    G3(((j-1)*m^2+(b-1)*m+a),:) = 2*mean((Crrg1)+(Crrg2))';   
    end;
  end;
end;

Sigma3  = (horzcat(sboots(K2+1:Tboots,:),guboots))'...
          *(horzcat(sboots(K2+1:Tboots,:),guboots))/(Tboots-K2);

O3 = [G3*cov,eye(K2*m^2)]*Sigma3*[G3*cov,eye(K2*m^2)]';                                 
T3 = T*(RRR*inv(O3)*RRR');      % Heteroskedasticity test statistic
