function [T1] = multintesttype1(r,rboots,sboots,gboots) 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MATLAB code: Section 11.6.1 (Quantile residuals)
% File: multintesttype1.m
% Coded by: JDG (converted from GAUSS code written by Leena Kalliovirta) 
%
% Normality test statistic based on MULTIVARIATE quantile residuals (QRs)
%    [TYPE 1 - (r^2-1)].
%               
% INPUT:     
% r      = vector of multivariate quantile residuals (T * m),
%          with m dimension of vector series.
% rboots = simulated vector of multivariate QRs using estimated model (Tboots * k) 
%          with k the number of columns of sboots.
% sboots = similated matrix of scores of QRs using estimated model 
%          (Tboots * k) with k the number of columns of sboots.                                 
% gboots = simulated matrix of derivatives of QRs using estimated models 
%          (Tboots * mk).                                    
%            
% OUTPUT: 
% T1     = scalar value of the test statistic, follows a Chi^{2}(3m) 
%          distribution under H0: no deviation from normal distribution
%          in multivariate quantile residuals.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
T      = length(r);
Tboots = length(rboots);
[~,k]  = size(sboots); 
[~,m]  = size(r);

warning('off','all') 
warning
% Disable all warnings. Replace 'off' by 'on' to enable
% Warning: Matrix "cov" may be close to singular or badly scaled.
%          Results may be inaccurate. 

cov = inv((sboots'*sboots)/Tboots);
% Matrix G1 has to be of size: ((m+4) * k)                                                 

mat1  = rboots(:,1);  
repr1 = repmat(mat1,1,k);              % repeat copies of array
C1    = repr1.*gboots(:,1:k);

mat2  = rboots(:,1).^2;  
repr2 = repmat(mat2,1,k);              % repeat copies of array
C2    = repr2.*gboots(:,1:k);

mat3  = rboots(:,1).^3;  
repr3 = repmat(mat3,1,k);              % repeat copies of array
C3    = repr3.*gboots(:,1:k);

G1 = [2*mean(C1);3*mean(C2);4*mean(C3)];
  
% Vector gu has to be: ((m+4) * 1)                                                 
gu      = [mean(r(:,1).^2-1);mean(r(:,1).^3);mean(r(:,1).^4-3)];
raboots = [(rboots(:,1).^2-1) (rboots(:,1).^3) (rboots(:,1).^4-3)];

for i=2:m
  mat1    = rboots(:,i);  
  repr1   = repmat(mat1,1,k);              % repeat copies of array
  C1      = repr1.*gboots(:,((i-1)*k+1):i*k);

  mat2    = rboots(:,i).^2;  
  repr2   = repmat(mat2,1,k);              % repeat copies of array
  C2      = repr2.*gboots(:,((i-1)*k+1):i*k);

  mat3    = rboots(:,i).^3;  
  repr3   = repmat(mat3,1,k);              % repeat copies of array
  C3      = repr3.*gboots(:,((i-1)*k+1):i*k);

  G1      = [G1;2*mean(C1);3*mean(C2);4*mean(C3)];
  gu      = [gu;mean(r(:,i).^2-1); mean(r(:,i).^3); mean(r(:,i).^4-3)];
  raboots = [raboots (rboots(:,i).^2-1) (rboots(:,i).^3) (rboots(:,i).^4-3)];
end;

Sigma = [sboots;raboots]'.*[sboots;raboots]/Tboots;
O1    = [G1*cov,eye(3*m)]*Sigma*[G1*cov,eye(3*m)]';
T1    = T*(gu'*inv(O1)*gu);
